import json
import os
from django.shortcuts import render, redirect, HttpResponse

from django.http import JsonResponse
from django.contrib import auth
from blog.utils.valid_code import get_valid_code
from blog.myforms import UserForm
from blog.models import *
from django.db.models import F
from django.db import transaction
from django.contrib.auth.decorators import login_required
from bbs_blog.settings import MEDIA_ROOT
from bs4 import BeautifulSoup


# Create your views here.


def login(request):
    if request.method == "POST":
        response = {'user': None, 'msg': None}
        user = request.POST.get("user")
        pwd = request.POST.get("pwd")
        verification = request.POST.get("verification")

        if verification.upper() == request.session.get("valid_code_str").upper():
            user = auth.authenticate(username=user, password=pwd)
            if user:
                auth.login(request, user)
                response["user"] = user.username
            else:
                response["msg"] = "用户名或密码错误"
        else:
            response["msg"] = "验证码错误"
        return JsonResponse(response)
    return render(request, "login.html")


def index(request):
    article_list = Article.objects.all()
    return render(request, "index.html", {"article_list": article_list})


def home_site(request, username, **kwargs):
    '''
    个人站点
    :param request: 
    :param path: 
    :return: 
    '''
    user = UserInfo.objects.filter(username=username).first()
    if user:
        blog = user.blog
        article_list = Article.objects.filter(user=user)
        if kwargs:
            condition = kwargs.get("condition")
            param = kwargs.get("param")
            if condition == "tag":
                article_list = article_list.filter(tags__title=param)
            elif condition == "category":
                article_list = article_list.filter(category__title=param)
            else:
                year, month = param.split("-")
                article_list = article_list.filter(create_time__year=year, create_time__month=month)

        return render(request, "home_site.html", {"blog": blog, "username": username, "article_list": article_list, })

    else:
        return render(request, "not_fount.html")


def article_detail(request, username, article_id):
    '''
    文章详情页
    :param request: 
    :param username: 
    :param article_id: 
    :return: 
    '''
    blog = Blog.objects.filter(userinfo__username=username).first()
    article_obj = Article.objects.filter(pk=article_id).first()

    comment_list = Comment.objects.filter(article_id=article_id)

    return render(request, "article_detail.html",
                  {"blog": blog, "username": username, "article_obj": article_obj, "comment_list": comment_list})


def digg(request):
    '''
    点赞
    :param request: 
    :return: 
    '''
    print(request.POST)
    article_id = request.POST.get("article_id")
    is_up = json.loads(request.POST.get("is_up"))
    user_id = request.user.pk

    response = {"state": True}
    updown = ArticleUpDown.objects.filter(article_id=article_id, user_id=user_id).first()
    if not updown:
        ArticleUpDown.objects.create(article_id=article_id, user_id=user_id, is_up=is_up)

        queryset = Article.objects.filter(pk=article_id)
        if is_up:
            queryset.update(up_count=F("up_count") + 1)
        else:
            queryset.update(down_count=F("down_count") + 1)
    else:
        response["state"] = False
        response["handle"] = updown.is_up

    return JsonResponse(response)


def comment(request):
    '''
    评论功能
    :param request: 
    :return: 
    '''
    article_id = request.POST.get("article_id")
    pid = request.POST.get("pid")
    content = request.POST.get("content")
    user_id = request.user.pk
    Article.objects.filter(pk=article_id).first()

    with transaction.atomic():
        comment_obj = Comment.objects.create(user_id=user_id, article_id=article_id, content=content,
                                             parent_comment_id=pid)
        Article.objects.filter(pk=article_id).update(comment_count=F("comment_count") + 1)

    response = {}
    response["create_time"] = comment_obj.create_time.strftime("%Y-%m-%d %X")
    response["username"] = request.user.username
    response["content"] = comment_obj.content

    return JsonResponse(response)


def get_comment_tree(request):
    article_id = request.GET.get("article_id")

    res = list(
        Comment.objects.filter(article_id=article_id).order_by("pk").values("pk", "content", "parent_comment_id"))
    print(res)
    return JsonResponse(res, safe=False)


def logout(request):
    '''
    注销
    :param request: 
    :return: 
    '''
    auth.logout(request)
    return redirect('/index/')


def register(request):
    if request.is_ajax():
        form = UserForm(request.POST)
        response = {"user": None, "msg": None}
        if form.is_valid():
            response["user"] = form.cleaned_data.get("user")
            user = form.cleaned_data.get("user")
            pwd = form.cleaned_data.get("pwd")
            email = form.cleaned_data.get("email")
            avatar_obj = request.FILES.get("avatar")

            extra = {}
            if avatar_obj:
                extra["avatar"] = avatar_obj

            UserInfo.objects.create_user(username=user, password=pwd, email=email, **extra)

        else:
            response["msg"] = form.errors
        return JsonResponse(response)

    form = UserForm()
    return render(request, "register.html", {"form": form})


def get_verification(request):
    data = get_valid_code(request)

    return HttpResponse(data)


@login_required
def cn_backend(request):
    article_list = Article.objects.filter(user=request.user)
    return render(request, "cn_backend/homepage.html", {"article_list": article_list})


@login_required
def add_article(request):
    if request.method == "POST":
        title = request.POST.get("title")
        content = request.POST.get("content")
        soup = BeautifulSoup(content, "html.parser")
        for tag in soup.find_all():
            if tag.name == "script":
                tag.decompose()
        desc = soup.text[0:150]

        Article.objects.create(title=title, desc=desc, content=str(soup), user=request.user)
        return redirect("/cn_backend/")

    return render(request, "cn_backend/add_article.html")


def upload(request):
    '''
    文件上传
    :param request: 
    :return: 
    '''
    img = request.FILES.get('upload_img')
    path = os.path.join(MEDIA_ROOT, "add_article_img", img.name)
    with open(path, "wb") as f:
        for line in img:
            f.write(line)

    # 固定格式
    response = {
        "error": 0,
        "url": "/media/add_article_img/%s" % img.name
    }

    return JsonResponse(response)
